@@ -97,6 +97,7 @@ public inline fun <T, K, R> Grouping<T, K>.fold(
         initialValueSelector: (key: K, element: T) -> R,
         operation: (key: K, accumulator: R, element: T) -> R
 ): Map<K, R> =
+        @Suppress("UNCHECKED_CAST")
         aggregate { key, acc, e, first -> operation(key, if (first) initialValueSelector(key, e) else acc as R, e) }
 
 /**
@@ -126,6 +127,7 @@ public inline fun <T, K, R, M : MutableMap<in K, R>> Grouping<T, K>.foldTo(
         initialValueSelector: (key: K, element: T) -> R,
         operation: (key: K, accumulator: R, element: T) -> R
 ): M =
+        @Suppress("UNCHECKED_CAST")
         aggregateTo(destination) { key, acc, e, first -> operation(key, if (first) initialValueSelector(key, e) else acc as R, e) }
 
 
@@ -145,6 +147,7 @@ public inline fun <T, K, R> Grouping<T, K>.fold(
         initialValue: R,
         operation: (accumulator: R, element: T) -> R
 ): Map<K, R> =
+        @Suppress("UNCHECKED_CAST")
         aggregate { _, acc, e, first -> operation(if (first) initialValue else acc as R, e) }
 
 /**
@@ -168,6 +171,7 @@ public inline fun <T, K, R, M : MutableMap<in K, R>> Grouping<T, K>.foldTo(
         initialValue: R,
         operation: (accumulator: R, element: T) -> R
 ): M =
+        @Suppress("UNCHECKED_CAST")
         aggregateTo(destination) { _, acc, e, first -> operation(if (first) initialValue else acc as R, e) }
 
 
@@ -190,6 +194,7 @@ public inline fun <S, T : S, K> Grouping<T, K>.reduce(
         operation: (key: K, accumulator: S, element: T) -> S
 ): Map<K, S> =
         aggregate { key, acc, e, first ->
+            @Suppress("UNCHECKED_CAST")
             if (first) e else operation(key, acc as S, e)
         }
 
@@ -216,25 +221,26 @@ public inline fun <S, T : S, K, M : MutableMap<in K, S>> Grouping<T, K>.reduceTo
         operation: (key: K, accumulator: S, element: T) -> S
 ): M =
         aggregateTo(destination) { key, acc, e, first ->
+            @Suppress("UNCHECKED_CAST")
             if (first) e else operation(key, acc as S, e)
         }
 
 
-/* Not available on platform: JS */
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+/**
+ * Groups elements from the [Grouping] source by key and counts elements in each group.
+ *
+ * @return a [Map] associating the key of each group with the count of elements in the group.
+ *
+ * @sample samples.collections.Collections.Transformations.groupingByEachCount
+ */
+@SinceKotlin("1.1")
+@JvmVersion
+public fun <T, K> Grouping<T, K>.eachCount(): Map<K, Int> =
+        // fold(0) { acc, e -> acc + 1 } optimized for boxing
+        foldTo( destination = mutableMapOf(),
+                initialValueSelector = { _, _ -> kotlin.jvm.internal.Ref.IntRef() },
+                operation = { _, acc, _ -> acc.apply { element += 1 } })
+        .mapValuesInPlace { it.value.element }
 
 /**
  * Groups elements from the [Grouping] source by key and counts elements in each group to the given [destination] map.
@@ -250,45 +256,46 @@ public inline fun <S, T : S, K, M : MutableMap<in K, S>> Grouping<T, K>.reduceTo
 public fun <T, K, M : MutableMap<in K, Int>> Grouping<T, K>.eachCountTo(destination: M): M =
         foldTo(destination, 0) { acc, _ -> acc + 1 }
 
-/* Not available on platform: JS */
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+/**
+/**
+ * Groups elements from the [Grouping] source by key and sums values provided by the [valueSelector] function for elements in each group.
+ *
+ * @return a [Map] associating the key of each group with the sum of elements in the group.
+ */
+@SinceKotlin("1.1")
+@JvmVersion
+public inline fun <T, K> Grouping<T, K>.eachSumOf(valueSelector: (T) -> Int): Map<K, Int> =
+        // fold(0) { acc, e -> acc + valueSelector(e)} optimized for boxing
+        foldTo( destination = mutableMapOf(),
+                initialValueSelector = { _, _ -> kotlin.jvm.internal.Ref.IntRef() },
+                operation = { _, acc, e -> acc.apply { element += valueSelector(e) } })
+        .mapValuesInPlace { it.value.element }
 
+/**
+ * Groups elements from the [Grouping] source by key and sums values provided by the [valueSelector] function for elements in each group
+ * to the given [destination] map.
+ *
+ *
+ * If the [destination] map already has a value corresponding to the key of some group,
+ * that value is used as an initial value of the sum for that group.
+ *
+ * @return the [destination] map associating the key of each group with the sum of elements in the group.
+ */
+@SinceKotlin("1.1")
+public inline fun <T, K, M : MutableMap<in K, Int>> Grouping<T, K>.eachSumOfTo(destination: M, valueSelector: (T) -> Int): M =
+        foldTo(destination, 0) { acc, e -> acc + valueSelector(e)}
+*/
 
+@JvmVersion
+@PublishedApi
+@kotlin.internal.InlineOnly
+@Suppress("UNCHECKED_CAST") // tricks with erased generics go here, do not repeat at reified platforms
+internal inline fun <K, V, R> MutableMap<K, V>.mapValuesInPlace(f: (Map.Entry<K, V>) -> R): MutableMap<K, R> {
+    entries.forEach {
+        (it as MutableMap.MutableEntry<K, R>).setValue(f(it))
+    }
+    return (this as MutableMap<K, R>)
+}
 
 /*
 // TODO: sum by long and by double overloads
