@@ -35,4 +35,13 @@ internal annotation class JvmMultifileClass
 @Target(AnnotationTarget.FIELD)
 @Retention(AnnotationRetention.SOURCE)
 @MustBeDocumented
-internal annotation class JvmField
\ No newline at end of file
+internal annotation class JvmField
+
+
+@Target(AnnotationTarget.FIELD)
+@Retention(AnnotationRetention.SOURCE)
+public annotation class Volatile
+
+@Target(AnnotationTarget.FUNCTION, AnnotationTarget.PROPERTY_GETTER, AnnotationTarget.PROPERTY_SETTER)
+@Retention(AnnotationRetention.SOURCE)
+public annotation class Synchronized
