@@ -29,6 +29,7 @@ import kotlin.reflect.jvm.internal.impl.util.capitalizeDecapitalize.CapitalizeDe
 
 import static kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils.isClassOrEnumClass;
 import static kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils.isCompanionObject;
+import static kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils.isInterface;
 
 public final class JvmAbi {
     public static final String DEFAULT_IMPLS_CLASS_NAME = "DefaultImpls";
@@ -54,6 +55,7 @@ public final class JvmAbi {
     private static final String ANNOTATED_TYPEALIAS_METHOD_NAME_SUFFIX = ANNOTATIONS_SUFFIX;
 
     public static final String INSTANCE_FIELD = "INSTANCE";
+    public static final String HIDDEN_INSTANCE_FIELD = "$$" + INSTANCE_FIELD;
 
     public static final String DEFAULT_MODULE_NAME = "main";
     public static final ClassId REFLECTION_FACTORY_IMPL = ClassId.topLevel(new FqName("kotlin.reflect.jvm.internal.ReflectionFactoryImpl"));
@@ -110,6 +112,16 @@ public final class JvmAbi {
     public static boolean isCompanionObjectWithBackingFieldsInOuter(@NotNull DeclarationDescriptor companionObject) {
         return isCompanionObject(companionObject) &&
                isClassOrEnumClass(companionObject.getContainingDeclaration()) &&
-               !CompanionObjectMapping.INSTANCE.isMappedIntrinsicCompanionObject((ClassDescriptor) companionObject);
+               !isMappedIntrinsicCompanionObject((ClassDescriptor) companionObject);
+    }
+
+    public static boolean isMappedIntrinsicCompanionObject(@NotNull ClassDescriptor companionObject) {
+        return CompanionObjectMapping.INSTANCE.isMappedIntrinsicCompanionObject(companionObject);
+    }
+
+    public static boolean isCompanionObjectInInterfaceNotIntrinsic(@NotNull DeclarationDescriptor companionObject) {
+        return isCompanionObject(companionObject) &&
+               isInterface(companionObject.getContainingDeclaration()) &&
+               !isMappedIntrinsicCompanionObject((ClassDescriptor) companionObject);
     }
 }
