@@ -16,11 +16,13 @@
 
 package kotlin.reflect.jvm.internal.impl.load.java.lazy.descriptors
 
+import kotlin.reflect.jvm.internal.impl.builtins.KotlinBuiltIns
 import kotlin.reflect.jvm.internal.impl.descriptors.*
 import kotlin.reflect.jvm.internal.impl.descriptors.impl.PropertyDescriptorImpl
 import kotlin.reflect.jvm.internal.impl.descriptors.impl.ValueParameterDescriptorImpl
 import kotlin.reflect.jvm.internal.impl.incremental.components.LookupLocation
 import kotlin.reflect.jvm.internal.impl.incremental.components.NoLookupLocation
+import kotlin.reflect.jvm.internal.impl.load.java.JvmAnnotationNames
 import kotlin.reflect.jvm.internal.impl.load.java.components.TypeUsage
 import kotlin.reflect.jvm.internal.impl.load.java.descriptors.JavaMethodDescriptor
 import kotlin.reflect.jvm.internal.impl.load.java.descriptors.JavaPropertyDescriptor
@@ -34,6 +36,7 @@ import kotlin.reflect.jvm.internal.impl.load.java.structure.JavaMethod
 import kotlin.reflect.jvm.internal.impl.load.java.structure.JavaValueParameter
 import kotlin.reflect.jvm.internal.impl.name.Name
 import kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils
+import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.firstArgumentValue
 import kotlin.reflect.jvm.internal.impl.resolve.retainMostSpecificInEachOverridableGroup
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.DescriptorKindExclude.NonExtensions
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.DescriptorKindFilter
@@ -45,6 +48,7 @@ import kotlin.reflect.jvm.internal.impl.types.KotlinType
 import kotlin.reflect.jvm.internal.impl.types.TypeUtils
 import kotlin.reflect.jvm.internal.impl.utils.Printer
 import kotlin.reflect.jvm.internal.impl.utils.addIfNotNull
+import kotlin.reflect.jvm.internal.impl.utils.addToStdlib.safeAs
 import java.util.*
 
 abstract class LazyJavaScope(protected val c: LazyJavaResolverContext) : MemberScopeImpl() {
@@ -63,6 +67,8 @@ abstract class LazyJavaScope(protected val c: LazyJavaResolverContext) : MemberS
 
     protected val declaredMemberIndex: NotNullLazyValue<DeclaredMemberIndex> = c.storageManager.createLazyValue { computeMemberIndex() }
 
+    fun wasContentRequested() = declaredMemberIndex.isComputed()
+
     protected abstract fun computeMemberIndex(): DeclaredMemberIndex
 
     // Fake overrides, values()/valueOf(), etc.
@@ -161,11 +167,16 @@ abstract class LazyJavaScope(protected val c: LazyJavaResolverContext) : MemberS
             jValueParameters: List<JavaValueParameter>
     ): ResolvedValueParameters {
         var synthesizedNames = false
-        val descriptors = jValueParameters.withIndex().map { pair ->
-            val (index, javaParameter) = pair
+        val usedNames = mutableSetOf<String>()
 
+        val descriptors = jValueParameters.withIndex().map { (index, javaParameter) ->
             val annotations = c.resolveAnnotations(javaParameter)
             val typeUsage = TypeUsage.COMMON.toAttributes()
+            val parameterName = annotations
+                    .findAnnotation(JvmAnnotationNames.PARAMETER_NAME_FQ_NAME)
+                    ?.firstArgumentValue()
+                    ?.safeAs<String>()
+
             val (outType, varargElementType) =
                     if (javaParameter.isVararg) {
                         val paramType = javaParameter.type as? JavaArrayType
@@ -186,6 +197,9 @@ abstract class LazyJavaScope(protected val c: LazyJavaResolverContext) : MemberS
                 // TODO: fix Java parameter name loading logic somehow (don't always load "p0", "p1", etc.)
                 Name.identifier("other")
             }
+            else if (parameterName != null && parameterName.isNotEmpty() && usedNames.add(parameterName)) {
+                Name.identifier(parameterName)
+            }
             else {
                 // TODO: parameter names may be drawn from attached sources, which is slow; it's better to make them lazy
                 val javaName = javaParameter.name
@@ -212,9 +226,15 @@ abstract class LazyJavaScope(protected val c: LazyJavaResolverContext) : MemberS
 
     private val functionNamesLazy by c.storageManager.createLazyValue { computeFunctionNames(DescriptorKindFilter.FUNCTIONS, null) }
     private val propertyNamesLazy by c.storageManager.createLazyValue { computePropertyNames(DescriptorKindFilter.VARIABLES, null) }
+    private val classNamesLazy by c.storageManager.createLazyValue { computeClassNames(DescriptorKindFilter.CLASSIFIERS, null) }
 
     override fun getFunctionNames() = functionNamesLazy
     override fun getVariableNames() = propertyNamesLazy
+    override fun getClassifierNames() = classNamesLazy
+
+    override fun definitelyDoesNotContainName(name: Name): Boolean {
+        return name !in functionNamesLazy && name !in propertyNamesLazy && name !in classNamesLazy
+    }
 
     override fun getContributedFunctions(name: Name, location: LookupLocation): Collection<SimpleFunctionDescriptor> {
         if (name !in getFunctionNames()) return emptyList()
@@ -280,12 +300,14 @@ abstract class LazyJavaScope(protected val c: LazyJavaResolverContext) : MemberS
     private fun getPropertyType(field: JavaField): KotlinType {
         // Fields do not have their own generic parameters.
         // Simple static constants should not have flexible types.
-        val isNotNullable = !(field.isFinalStatic && field.hasConstantNotNullInitializer)
         val propertyType = c.typeResolver.transformJavaType(
                 field.type,
                 TypeUsage.COMMON.toAttributes()
         )
-        if (!isNotNullable) {
+        val isNotNullable =
+                (KotlinBuiltIns.isPrimitiveType(propertyType) || KotlinBuiltIns.isString(propertyType)) &&
+                field.isFinalStatic && field.hasConstantNotNullInitializer
+        if (isNotNullable) {
             return TypeUtils.makeNotNullable(propertyType)
         }
 
