@@ -31,13 +31,10 @@ import kotlin.reflect.jvm.internal.impl.load.kotlin.computeJvmDescriptor
 import kotlin.reflect.jvm.internal.impl.name.FqName
 import kotlin.reflect.jvm.internal.impl.platform.JavaToKotlinClassMap
 import kotlin.reflect.jvm.internal.impl.resolve.descriptorUtil.firstArgumentValue
-import kotlin.reflect.jvm.internal.impl.types.KotlinType
-import kotlin.reflect.jvm.internal.impl.types.TypeUtils
-import kotlin.reflect.jvm.internal.impl.types.asFlexibleType
+import kotlin.reflect.jvm.internal.impl.types.*
 import kotlin.reflect.jvm.internal.impl.types.checker.KotlinTypeChecker
-import kotlin.reflect.jvm.internal.impl.types.isFlexible
 import kotlin.reflect.jvm.internal.impl.types.typeUtil.isTypeParameter
-import kotlin.reflect.jvm.internal.impl.types.unwrapEnhancement
+import kotlin.reflect.jvm.internal.impl.utils.Jsr305State
 import kotlin.reflect.jvm.internal.impl.utils.addToStdlib.firstNotNullResult
 import kotlin.reflect.jvm.internal.impl.utils.addToStdlib.safeAs
 
@@ -46,7 +43,10 @@ data class NullabilityQualifierWithMigrationStatus(
         val isForWarningOnly: Boolean = false
 )
 
-class SignatureEnhancement(private val annotationTypeQualifierResolver: AnnotationTypeQualifierResolver) {
+class SignatureEnhancement(
+    private val annotationTypeQualifierResolver: AnnotationTypeQualifierResolver,
+    private val jsr305State: Jsr305State
+) {
 
     private fun AnnotationDescriptor.extractNullabilityTypeFromArgument(): NullabilityQualifierWithMigrationStatus? {
         val enumEntryDescriptor = firstArgumentValue()
@@ -81,10 +81,16 @@ class SignatureEnhancement(private val annotationTypeQualifierResolver: Annotati
     ): NullabilityQualifierWithMigrationStatus? {
         val annotationFqName = annotationDescriptor.fqName ?: return null
 
-        return when (annotationFqName) {
-            in NULLABLE_ANNOTATIONS -> NullabilityQualifierWithMigrationStatus(NullabilityQualifier.NULLABLE)
-            in NOT_NULL_ANNOTATIONS -> NullabilityQualifierWithMigrationStatus(NullabilityQualifier.NOT_NULL)
-            JAVAX_NONNULL_ANNOTATION -> annotationDescriptor.extractNullabilityTypeFromArgument()
+        return when {
+            annotationFqName in NULLABLE_ANNOTATIONS -> NullabilityQualifierWithMigrationStatus(NullabilityQualifier.NULLABLE)
+            annotationFqName in NOT_NULL_ANNOTATIONS -> NullabilityQualifierWithMigrationStatus(NullabilityQualifier.NOT_NULL)
+            annotationFqName == JAVAX_NONNULL_ANNOTATION -> annotationDescriptor.extractNullabilityTypeFromArgument()
+
+            annotationFqName == COMPATQUAL_NULLABLE_ANNOTATION && jsr305State.enableCompatqualCheckerFrameworkAnnotations ->
+                NullabilityQualifierWithMigrationStatus(NullabilityQualifier.NULLABLE)
+
+            annotationFqName == COMPATQUAL_NONNULL_ANNOTATION && jsr305State.enableCompatqualCheckerFrameworkAnnotations ->
+                NullabilityQualifierWithMigrationStatus(NullabilityQualifier.NOT_NULL)
             else -> null
         }
     }
