@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2016 JetBrains s.r.o.
+ * Copyright 2010-2017 JetBrains s.r.o.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,13 +20,16 @@ import kotlin.reflect.jvm.internal.impl.builtins.KotlinBuiltIns
 import kotlin.reflect.jvm.internal.impl.descriptors.*
 import kotlin.reflect.jvm.internal.impl.descriptors.ClassKind.*
 import kotlin.reflect.jvm.internal.impl.descriptors.annotations.Annotated
+import kotlin.reflect.jvm.internal.impl.descriptors.annotations.AnnotationDescriptor
 import kotlin.reflect.jvm.internal.impl.descriptors.annotations.KotlinRetention
 import kotlin.reflect.jvm.internal.impl.descriptors.impl.TypeAliasConstructorDescriptor
 import kotlin.reflect.jvm.internal.impl.incremental.components.LookupLocation
 import kotlin.reflect.jvm.internal.impl.name.ClassId
 import kotlin.reflect.jvm.internal.impl.name.FqName
 import kotlin.reflect.jvm.internal.impl.name.FqNameUnsafe
+import kotlin.reflect.jvm.internal.impl.name.Name
 import kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils
+import kotlin.reflect.jvm.internal.impl.resolve.DescriptorUtils.getContainingClass
 import kotlin.reflect.jvm.internal.impl.resolve.constants.EnumValue
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.DescriptorKindFilter
 import kotlin.reflect.jvm.internal.impl.resolve.scopes.MemberScope
@@ -39,6 +42,7 @@ import kotlin.reflect.jvm.internal.impl.types.typeUtil.isAnyOrNullableAny
 import kotlin.reflect.jvm.internal.impl.types.typeUtil.makeNullable
 import kotlin.reflect.jvm.internal.impl.utils.DFS
 import kotlin.reflect.jvm.internal.impl.utils.SmartList
+import kotlin.reflect.jvm.internal.impl.utils.addToStdlib.safeAs
 
 fun ClassDescriptor.getClassObjectReferenceTarget(): ClassDescriptor = companionObjectDescriptor ?: this
 
@@ -211,10 +215,10 @@ fun Annotated.isDocumentedAnnotation(): Boolean =
         annotations.findAnnotation(KotlinBuiltIns.FQ_NAMES.mustBeDocumented) != null
 
 fun Annotated.getAnnotationRetention(): KotlinRetention? {
-    val annotationEntryDescriptor = annotations.findAnnotation(KotlinBuiltIns.FQ_NAMES.retention) ?: return null
-    val retentionArgumentValue = annotationEntryDescriptor.allValueArguments.entries.firstOrNull {
-        it.key.name.asString() == "value"
-    }?.value as? EnumValue ?: return null
+    val retentionArgumentValue = annotations.findAnnotation(KotlinBuiltIns.FQ_NAMES.retention)
+                                         ?.allValueArguments
+                                         ?.get(Name.identifier("value"))
+                                         as? EnumValue ?: return null
     return KotlinRetention.valueOf(retentionArgumentValue.value.name.asString())
 }
 
@@ -401,3 +405,37 @@ fun DeclarationDescriptor.isPublishedApi(): Boolean {
     val descriptor = if (this is CallableMemberDescriptor) DescriptorUtils.getDirectMember(this) else this
     return descriptor.annotations.hasAnnotation(KotlinBuiltIns.FQ_NAMES.publishedApi)
 }
+
+fun DeclarationDescriptor.isAncestorOf(descriptor: DeclarationDescriptor, strict: Boolean): Boolean
+        = DescriptorUtils.isAncestor(this, descriptor, strict)
+
+fun DeclarationDescriptor.isCompanionObject(): Boolean = DescriptorUtils.isCompanionObject(this)
+
+fun ClassDescriptor.isSubclassOf(superclass: ClassDescriptor): Boolean = DescriptorUtils.isSubclass(this, superclass)
+
+val AnnotationDescriptor.annotationClass: ClassDescriptor?
+    get() = type.constructor.declarationDescriptor as? ClassDescriptor
+
+fun AnnotationDescriptor.firstArgumentValue() = allValueArguments.values.firstOrNull()?.value
+
+fun MemberDescriptor.isEffectivelyExternal(): Boolean {
+    if (isExternal) return true
+
+    if (this is PropertyAccessorDescriptor) {
+        val variableDescriptor = correspondingProperty
+        if (variableDescriptor.isEffectivelyExternal()) return true
+    }
+
+    if (this is PropertyDescriptor) {
+        if (getter?.isExternal == true &&
+            (!isVar || setter?.isExternal == true)) return true
+    }
+
+    val containingClass = getContainingClass(this)
+    return containingClass != null && containingClass.isEffectivelyExternal()
+}
+
+fun isParameterOfAnnotation(parameterDescriptor: ParameterDescriptor): Boolean {
+    val constructedClass = parameterDescriptor.containingDeclaration.safeAs<ConstructorDescriptor>()?.constructedClass
+    return DescriptorUtils.isAnnotationClass(constructedClass)
+}
